# $Id: ExoRedirectTool.py,v 1.4 2001/06/19 08:19:51 bjorn Exp $
__doc__ = """Redirect tool

 Rewrites URL requests based on a list of match patterns
 and replacements, much similar to Apache Rewrite.
 Each match/replacement is stored in an ExoRedirect object
 which resides in the tool.

 Redirection is done by hooking into the traversal engine
 (ExoRedirectTool.__call__) and having it publish a method
 that does the action redirect (ExoRedirectTool.redir).
 This indirect method is necessary since the traversal
 engine catches all exceptions.

 TODO:
 - Does not work for virtual hosting
 - Could be combined with portal_url tool
"""
__rcs_id__ = '$Id: ExoRedirectTool.py,v 1.4 2001/06/19 08:19:51 bjorn Exp $'
__version__ = '$Revision: 1.4 $'[11:-2]

import re
from string import split, join
from whrandom import randint
from DateTime import DateTime
from Globals import InitializeClass, HTMLFile, HTML, MessageDialog
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from Products.CMFCore.PortalFolder import PortalFolder
from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFCore.CMFCorePermissions import *
from ZPublisher import BeforeTraverse


class ExoRedirectTool(UniqueObject, PortalFolder):
    """This tool implements URL redirection rules.
    """
    id = 'portal_redirect'
    title = ''
    meta_type = 'Redirect Tool'
    security = ClassSecurityInfo()

    status_codes = (
        'Moved Permanently',        # 301
        'Moved Temporarily',        # 302
        'See Other',                # 303
        'Temporary Redirect',       # 307
        'Unauthorized',             # 401
        'Payment Required',         # 402
        'Forbidden',                # 403
        'Not Found',                # 404
        'Gone',                     # 410
        'Not Implemented',          # 501
        'Service Unavailable')      # 503

    all_meta_types = ({'name':'Redirect',
                       'action':'manage_addExoRedirectForm'})
    manage_options = PortalFolder.manage_options

    def __init__(self):
        pass

    def __call__(self, container, REQUEST):
        """Hooks the traversal path."""
        #
        # Convert the URLs to make it relative to
        # this tool's parent's URL
        #
        # TODO:
        # - doesn't work for virtual hosting
        #
        base_url = REQUEST.physicalPathToURL(container.getPhysicalPath(),
                                             relative=1) + '/'
        url = REQUEST['PATH_INFO']
        relative_url = url[len(base_url):]
        
        #
        # Try to rewrite the remaining url
        #
        redir = self.rewrite(relative_url)
        if redir:
            #
            # Now make the traversal engine publish our redir method
            # with the new url and status pushed unto the REQUEST.
            # Note that we have to do the redirection indirectly
            # through another function since the traversal engine
            # catches all exceptions (and a redirect is an exception).
            #
            (status, new_url) = redir
            REQUEST['url'] = new_url
            REQUEST['status'] = status
            REQUEST['TraversalRequestNameStack'] = ['redir', self.id]

    security.declarePublic('redir')
    def redir(self, url, status, REQUEST):
        """ """
        return REQUEST.RESPONSE.redirect(url, status)

    security.declareProtected(ModifyPortalContent, 'add')
    def add(self, REQUEST):
        """ """
        manage_addExoRedirect(self, REQUEST=REQUEST)
        return REQUEST.RESPONSE.redirect(self.absolute_url()+'/editForm')

    security.declareProtected(ModifyPortalContent, 'edit')
    def edit(self, REQUEST):
        """ """
        for r in self.getRedirects():
            try:
                match = REQUEST[r.id+'_match_pattern']
                replace = REQUEST[r.id+'_replace_pattern']
                status = REQUEST[r.id+'_status_code']
                if match != r.match_pattern or replace != r.replace_pattern \
                   or status != r.status_code:
                    r.edit(match, replace, status)
            except:
                pass    # item must've been removed
        return REQUEST.RESPONSE.redirect(self.absolute_url()+'/editForm')

    security.declareProtected(ModifyPortalContent, 'editForm')
    editForm = HTMLFile('editForm', globals())
    manage_main = manage_workspace = editForm

    security.declarePublic('rewrite')
    def rewrite(self, old_url):
        """ """
        for r in self.getRedirects():
            (status, new_url) = r.rewrite(old_url)
            if new_url != old_url:
                return (status, new_url)
        return None

    security.declarePublic('getRedirects')
    def getRedirects(self):
        """ """
        return self.objectValues(['Redirect'])

InitializeClass(ExoRedirectTool)

manage_addExoRedirectToolForm = HTMLFile('addExoRedirectTool', globals())

def manage_addExoRedirectTool(self, id=None, REQUEST=None):
    """Create a redirect tool and install it in its parent Folder.
    The argument 'self' will be bound to the parent Folder.
    (TODO: should always install in portal root)
    """
    o = ExoRedirectTool()
    self._setObject(o.id, o)

    hook = BeforeTraverse.NameCaller(o.id)
    BeforeTraverse.registerBeforeTraverse(self.this(),
                                          hook,
                                          o.meta_type)
    return MessageDialog(
        title='Redirect tool created',
        message='Redirect tool created',
        action='./manage_main')


#####################################################


class ExoRedirect(SimpleItem):
    """Redirects reside inside of the redirect tool.
    """
    meta_type = 'Redirect'

    security = ClassSecurityInfo()
    #manage_options = SimpleItem.manage_options

    match_pattern = replace_pattern = status_code = None

    def __init__(self, id, match_pattern, replace_pattern, status_code):
        self.id = id
        self.edit(match_pattern, replace_pattern, status_code)

    security.declareProtected(ModifyPortalContent, 'edit')
    def edit(self, match_pattern, replace_pattern, status_code):
        self.match_pattern = match_pattern
        self.replace_pattern = replace_pattern
        self.status_code = status_code

    security.declarePublic('rewrite')
    def rewrite(self, url):
        """Rewrite and return the rewritten URL.
        If the URL doesn't match match_pattern, just return it untouched.
        """
        return (self.status_code,
                re.sub(self.match_pattern, self.replace_pattern, url))
        
InitializeClass(ExoRedirect)

manage_addExoRedirectForm = HTMLFile('addExoRedirect', globals())

def manage_addExoRedirect(self, id=None, REQUEST=None):
    """Create a redirect object its parent Folder.
    The argument 'self' will be bound to the parent Folder.
    (TODO: should always install in redirecting tool)
    """
    id = str(randint(0, 999999999))
    o = ExoRedirect(id, REQUEST['match_pattern'], REQUEST['replace_pattern'], REQUEST['status_code'])
    self._setObject(id, o)
